@@ -2,7 +2,6 @@ import unittest
 
 
 class TestRenderSelect(unittest.TestCase):
-
     def test_multiple_selects(self):
         """Ensure that render select can handle multiple selects."""
         from bigquery.query_builder import _render_select
@@ -30,7 +29,7 @@ class TestRenderSelect(unittest.TestCase):
 
         result = _render_select({
             'start_time': {'alias': 'TimeStamp', 'format':
-                           'SEC_TO_MICRO-INTEGER-FORMAT_UTC_USEC'}
+                'SEC_TO_MICRO-INTEGER-FORMAT_UTC_USEC'}
         })
 
         self.assertEqual(
@@ -47,7 +46,6 @@ class TestRenderSelect(unittest.TestCase):
 
 
 class TestRenderSources(unittest.TestCase):
-
     def test_multi_tables(self):
         """Ensure that render sources can handle multiple sources."""
         from bigquery.query_builder import _render_sources
@@ -75,7 +73,6 @@ class TestRenderSources(unittest.TestCase):
 
 
 class TestRenderConditions(unittest.TestCase):
-
     def test_single_condition(self):
         """Ensure that render conditions can handle a single condition."""
         from bigquery.query_builder \
@@ -152,9 +149,65 @@ class TestRenderConditions(unittest.TestCase):
                                  "STRING('d') AND NOT foobar == STRING('e') "
                                  "AND NOT foobar <= STRING('f')))")
 
+    def test_boolean_field_type(self):
+        """Ensure that render conditions can handle a boolean field."""
+        from bigquery.query_builder \
+            import _render_conditions
+
+        result = _render_conditions([
+            {
+                'field': 'foobar',
+                'type': 'BOOLEAN',
+                'comparators': [
+                    {'condition': '==', 'negate': False, 'value': True},
+                    {'condition': '!=', 'negate': False, 'value': False},
+                    {'condition': '==', 'negate': False, 'value': 'a'},
+                    {'condition': '!=', 'negate': False, 'value': ''},
+                    {'condition': '==', 'negate': True, 'value': 100},
+                    {'condition': '!=', 'negate': True, 'value': 0},
+                ]
+            }
+        ])
+
+        self.assertEqual(result, "WHERE ((foobar == BOOLEAN(1) AND "
+                                 "foobar != BOOLEAN(0) AND foobar == "
+                                 "BOOLEAN(1) AND foobar != BOOLEAN(0)) AND "
+                                 "(NOT foobar == BOOLEAN(1) AND NOT "
+                                 "foobar != BOOLEAN(0)))")
+
+    def test_in_comparator(self):
+        """Ensure that render conditions can handle "IN" condition."""
+        from bigquery.query_builder \
+            import _render_conditions
+
+        result = _render_conditions([
+            {
+                'field': 'foobar',
+                'type': 'STRING',
+                'comparators': [
+                    {'condition': 'IN', 'negate': False, 'value': ['a', 'b']},
+                    {'condition': 'IN', 'negate': False, 'value': {'c', 'd'}},
+                    {'condition': 'IN', 'negate': False, 'value': ('e', 'f')},
+                    {'condition': 'IN', 'negate': False, 'value': 'g'},
+                    {'condition': 'IN', 'negate': True, 'value': ['h', 'i']},
+                    {'condition': 'IN', 'negate': True, 'value': {'j', 'k'}},
+                    {'condition': 'IN', 'negate': True, 'value': ('l', 'm')},
+                    {'condition': 'IN', 'negate': True, 'value': 'n'},
+                ]
+            }
+        ])
+
+        self.assertEqual(result, "WHERE ((foobar IN (STRING('a'), STRING('b'))"
+                                 " AND foobar IN (STRING('c'), STRING('d')) "
+                                 "AND foobar IN (STRING('e'), STRING('f')) AND"
+                                 " foobar IN (STRING('g'))) AND (NOT foobar IN"
+                                 " (STRING('h'), STRING('i')) AND NOT foobar "
+                                 "IN (STRING('k'), STRING('j')) AND NOT foobar"
+                                 " IN (STRING('l'), STRING('m')) AND NOT "
+                                 "foobar IN (STRING('n'))))")
+
 
 class TestRenderOrder(unittest.TestCase):
-
     def test_order(self):
         """Ensure that render order can work under expected conditions."""
         from bigquery.query_builder import _render_order
@@ -173,7 +226,6 @@ class TestRenderOrder(unittest.TestCase):
 
 
 class TestGroupings(unittest.TestCase):
-
     def test_mutliple_fields(self):
         """Ensure that render grouping works with multiple fields."""
         from bigquery.query_builder \
@@ -194,7 +246,6 @@ class TestGroupings(unittest.TestCase):
 
 
 class TestRenderQuery(unittest.TestCase):
-
     def test_full_query(self):
         """Ensure that all the render query arguments work together."""
         from bigquery.query_builder import render_query
@@ -278,7 +329,7 @@ class TestRenderQuery(unittest.TestCase):
             },
             conditions=[
                 {'asdfasdfasdf': 'start_time', 'ffd': 1371566954, 'comparator':
-                 '<=', 'type': 'INTEGER'},
+                    '<=', 'type': 'INTEGER'},
                 {'field': 'start_time', 'value': {'value': 1371556954,
                                                   'negate': False},
                  'compoorattor': '>=', 'type': 'INTEGER'}
@@ -526,7 +577,7 @@ class TestRenderQuery(unittest.TestCase):
             tables=['2013_06_appspot_1'],
             select={
                 'start_time': {'alias': 'timestamp', 'format':
-                               'INTEGER-FORMAT_UTC_USEC'},
+                    'INTEGER-FORMAT_UTC_USEC'},
                 'status': {'alias': 'status'},
                 'resource': {'alias': 'url'}
             },
@@ -561,10 +612,10 @@ class TestRenderQuery(unittest.TestCase):
             tables=['2013_06_appspot_1'],
             select={
                 'start_time': [{'alias': 'timestamp', 'format':
-                                'INTEGER-FORMAT_UTC_USEC'},
+                    'INTEGER-FORMAT_UTC_USEC'},
                                {'alias': 'day', 'format':
-                                'SEC_TO_MICRO-INTEGER-FORMAT_UTC_USEC-LEFT:10'}
-                               ],
+                                   'SEC_TO_MICRO-INTEGER-FORMAT_UTC_USEC-LEFT:10'}
+                ],
                 'status': {'alias': 'status'},
                 'resource': {'alias': 'url'}
             },
@@ -601,7 +652,7 @@ class TestRenderQuery(unittest.TestCase):
             tables=['2013_06_appspot_1'],
             select={
                 'start_time': {'alias': 'timestamp', 'format':
-                               'SEC_TO_MICRO-INTEGER-SEC_TO_TIMESTAMP'},
+                    'SEC_TO_MICRO-INTEGER-SEC_TO_TIMESTAMP'},
                 'status': {'alias': 'status'},
                 'resource': {'alias': 'url'}
             },
@@ -707,4 +758,3 @@ class TestRenderQuery(unittest.TestCase):
                           "INTEGER('1371556954')) GROUP BY timestamp, status "
                           "ORDER BY timestamp desc")
         self.assertEqual(result, expected_query)
-
